@@ -2,22 +2,22 @@
 #          Joris Van den Bossche <jorisvandenbossche@gmail.com>
 # License: BSD 3 clause
 
+import warnings
 import numpy as np
 from scipy import sparse
 import numbers
 
 from ..base import BaseEstimator, TransformerMixin
 from ..utils import check_array, is_scalar_nan
+from ..utils.deprecation import deprecated
 from ..utils.validation import check_is_fitted
-from ..utils.validation import _deprecate_positional_args
+from ..utils.validation import _check_feature_names_in
+from ..utils._mask import _get_mask
 
 from ..utils._encode import _encode, _check_unknown, _unique
 
 
-__all__ = [
-    'OneHotEncoder',
-    'OrdinalEncoder'
-]
+__all__ = ["OneHotEncoder", "OrdinalEncoder"]
 
 
 class _BaseEncoder(TransformerMixin, BaseEstimator):
@@ -36,17 +36,14 @@ class _BaseEncoder(TransformerMixin, BaseEstimator):
         - return list of features (arrays): this list of features is
           constructed feature by feature to preserve the data types
           of pandas DataFrame columns, as otherwise information is lost
-          and cannot be used, eg for the `categories_` attribute.
+          and cannot be used, e.g. for the `categories_` attribute.
 
         """
-        if not (hasattr(X, 'iloc') and getattr(X, 'ndim', 0) == 2):
+        if not (hasattr(X, "iloc") and getattr(X, "ndim", 0) == 2):
             # if not a dataframe, do normal check_array validation
-            X_temp = check_array(X, dtype=None,
-                                 force_all_finite=force_all_finite)
-            if (not hasattr(X, 'dtype')
-                    and np.issubdtype(X_temp.dtype, np.str_)):
-                X = check_array(X, dtype=object,
-                                force_all_finite=force_all_finite)
+            X_temp = check_array(X, dtype=None, force_all_finite=force_all_finite)
+            if not hasattr(X, "dtype") and np.issubdtype(X_temp.dtype, np.str_):
+                X = check_array(X, dtype=object, force_all_finite=force_all_finite)
             else:
                 X = X_temp
             needs_validation = False
@@ -60,109 +57,127 @@ class _BaseEncoder(TransformerMixin, BaseEstimator):
 
         for i in range(n_features):
             Xi = self._get_feature(X, feature_idx=i)
-            Xi = check_array(Xi, ensure_2d=False, dtype=None,
-                             force_all_finite=needs_validation)
+            Xi = check_array(
+                Xi, ensure_2d=False, dtype=None, force_all_finite=needs_validation
+            )
             X_columns.append(Xi)
 
         return X_columns, n_samples, n_features
 
     def _get_feature(self, X, feature_idx):
-        if hasattr(X, 'iloc'):
+        if hasattr(X, "iloc"):
             # pandas dataframes
             return X.iloc[:, feature_idx]
         # numpy arrays, sparse arrays
         return X[:, feature_idx]
 
-    def _fit(self, X, handle_unknown='error', force_all_finite=True):
+    def _fit(self, X, handle_unknown="error", force_all_finite=True):
+        self._check_n_features(X, reset=True)
+        self._check_feature_names(X, reset=True)
         X_list, n_samples, n_features = self._check_X(
-            X, force_all_finite=force_all_finite)
+            X, force_all_finite=force_all_finite
+        )
+        self.n_features_in_ = n_features
 
-        if self.categories != 'auto':
+        if self.categories != "auto":
             if len(self.categories) != n_features:
-                raise ValueError("Shape mismatch: if categories is an array,"
-                                 " it has to be of shape (n_features,).")
+                raise ValueError(
+                    "Shape mismatch: if categories is an array,"
+                    " it has to be of shape (n_features,)."
+                )
 
         self.categories_ = []
 
         for i in range(n_features):
             Xi = X_list[i]
-            if self.categories == 'auto':
+            if self.categories == "auto":
                 cats = _unique(Xi)
             else:
                 cats = np.array(self.categories[i], dtype=Xi.dtype)
-                if Xi.dtype.kind not in 'OUS':
+                if Xi.dtype.kind not in "OUS":
                     sorted_cats = np.sort(cats)
-                    error_msg = ("Unsorted categories are not "
-                                 "supported for numerical categories")
+                    error_msg = (
+                        "Unsorted categories are not supported for numerical categories"
+                    )
                     # if there are nans, nan should be the last element
                     stop_idx = -1 if np.isnan(sorted_cats[-1]) else None
-                    if (np.any(sorted_cats[:stop_idx] != cats[:stop_idx]) or
-                        (np.isnan(sorted_cats[-1]) and
-                         not np.isnan(sorted_cats[-1]))):
+                    if np.any(sorted_cats[:stop_idx] != cats[:stop_idx]) or (
+                        np.isnan(sorted_cats[-1]) and not np.isnan(sorted_cats[-1])
+                    ):
                         raise ValueError(error_msg)
 
-                if handle_unknown == 'error':
+                if handle_unknown == "error":
                     diff = _check_unknown(Xi, cats)
                     if diff:
-                        msg = ("Found unknown categories {0} in column {1}"
-                               " during fit".format(diff, i))
+                        msg = (
+                            "Found unknown categories {0} in column {1}"
+                            " during fit".format(diff, i)
+                        )
                         raise ValueError(msg)
             self.categories_.append(cats)
 
-    def _transform(self, X, handle_unknown='error', force_all_finite=True):
+    def _transform(
+        self, X, handle_unknown="error", force_all_finite=True, warn_on_unknown=False
+    ):
+        self._check_feature_names(X, reset=False)
+        self._check_n_features(X, reset=False)
         X_list, n_samples, n_features = self._check_X(
-            X, force_all_finite=force_all_finite)
+            X, force_all_finite=force_all_finite
+        )
 
         X_int = np.zeros((n_samples, n_features), dtype=int)
         X_mask = np.ones((n_samples, n_features), dtype=bool)
 
-        if n_features != len(self.categories_):
-            raise ValueError(
-                "The number of features in X is different to the number of "
-                "features of the fitted data. The fitted data had {} features "
-                "and the X has {} features."
-                .format(len(self.categories_,), n_features)
-            )
-
+        columns_with_unknown = []
         for i in range(n_features):
             Xi = X_list[i]
-            diff, valid_mask = _check_unknown(Xi, self.categories_[i],
-                                              return_mask=True)
+            diff, valid_mask = _check_unknown(Xi, self.categories_[i], return_mask=True)
 
             if not np.all(valid_mask):
-                if handle_unknown == 'error':
-                    msg = ("Found unknown categories {0} in column {1}"
-                           " during transform".format(diff, i))
+                if handle_unknown == "error":
+                    msg = (
+                        "Found unknown categories {0} in column {1}"
+                        " during transform".format(diff, i)
+                    )
                     raise ValueError(msg)
                 else:
+                    if warn_on_unknown:
+                        columns_with_unknown.append(i)
                     # Set the problematic rows to an acceptable value and
                     # continue `The rows are marked `X_mask` and will be
                     # removed later.
                     X_mask[:, i] = valid_mask
                     # cast Xi into the largest string type necessary
                     # to handle different lengths of numpy strings
-                    if (self.categories_[i].dtype.kind in ('U', 'S')
-                            and self.categories_[i].itemsize > Xi.itemsize):
+                    if (
+                        self.categories_[i].dtype.kind in ("U", "S")
+                        and self.categories_[i].itemsize > Xi.itemsize
+                    ):
                         Xi = Xi.astype(self.categories_[i].dtype)
-                    elif (self.categories_[i].dtype.kind == 'O' and
-                            Xi.dtype.kind == 'U'):
+                    elif self.categories_[i].dtype.kind == "O" and Xi.dtype.kind == "U":
                         # categories are objects and Xi are numpy strings.
                         # Cast Xi to an object dtype to prevent truncation
                         # when setting invalid values.
-                        Xi = Xi.astype('O')
+                        Xi = Xi.astype("O")
                     else:
                         Xi = Xi.copy()
 
                     Xi[~valid_mask] = self.categories_[i][0]
             # We use check_unknown=False, since _check_unknown was
             # already called above.
-            X_int[:, i] = _encode(Xi, uniques=self.categories_[i],
-                                  check_unknown=False)
+            X_int[:, i] = _encode(Xi, uniques=self.categories_[i], check_unknown=False)
+        if columns_with_unknown:
+            warnings.warn(
+                "Found unknown categories in columns "
+                f"{columns_with_unknown} during transform. These "
+                "unknown categories will be encoded as all zeros",
+                UserWarning,
+            )
 
         return X_int, X_mask
 
     def _more_tags(self):
-        return {'X_types': ['categorical']}
+        return {"X_types": ["categorical"]}
 
 
 class OneHotEncoder(_BaseEncoder):
@@ -203,7 +218,7 @@ class OneHotEncoder(_BaseEncoder):
 
         .. versionadded:: 0.20
 
-    drop : {'first', 'if_binary'} or a array-like of shape (n_features,), \
+    drop : {'first', 'if_binary'} or an array-like of shape (n_features,), \
             default=None
         Specifies a methodology to use to drop one of the categories per
         feature. This is useful in situations where perfectly collinear
@@ -263,6 +278,17 @@ class OneHotEncoder(_BaseEncoder):
         .. versionchanged:: 0.23
            Added the possibility to contain `None` values.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 1.0
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
     See Also
     --------
     OrdinalEncoder : Performs an ordinal (integer)
@@ -298,9 +324,8 @@ class OneHotEncoder(_BaseEncoder):
     >>> enc.inverse_transform([[0, 1, 1, 0, 0], [0, 0, 0, 1, 0]])
     array([['Male', 1],
            [None, 2]], dtype=object)
-    >>> enc.get_feature_names(['gender', 'group'])
-    array(['gender_Female', 'gender_Male', 'group_1', 'group_2', 'group_3'],
-      dtype=object)
+    >>> enc.get_feature_names_out(['gender', 'group'])
+    array(['gender_Female', 'gender_Male', 'group_1', 'group_2', 'group_3'], ...)
 
     One can always drop the first column for each feature:
 
@@ -319,9 +344,15 @@ class OneHotEncoder(_BaseEncoder):
            [1., 0., 1., 0.]])
     """
 
-    @_deprecate_positional_args
-    def __init__(self, *, categories='auto', drop=None, sparse=True,
-                 dtype=np.float64, handle_unknown='error'):
+    def __init__(
+        self,
+        *,
+        categories="auto",
+        drop=None,
+        sparse=True,
+        dtype=np.float64,
+        handle_unknown="error",
+    ):
         self.categories = categories
         self.sparse = sparse
         self.dtype = dtype
@@ -329,33 +360,30 @@ class OneHotEncoder(_BaseEncoder):
         self.drop = drop
 
     def _validate_keywords(self):
-        if self.handle_unknown not in ('error', 'ignore'):
-            msg = ("handle_unknown should be either 'error' or 'ignore', "
-                   "got {0}.".format(self.handle_unknown))
+        if self.handle_unknown not in ("error", "ignore"):
+            msg = (
+                "handle_unknown should be either 'error' or 'ignore', got {0}.".format(
+                    self.handle_unknown
+                )
+            )
             raise ValueError(msg)
-        # If we have both dropped columns and ignored unknown
-        # values, there will be ambiguous cells. This creates difficulties
-        # in interpreting the model.
-        if self.drop is not None and self.handle_unknown != 'error':
-            raise ValueError(
-                "`handle_unknown` must be 'error' when the drop parameter is "
-                "specified, as both would create categories that are all "
-                "zero.")
 
     def _compute_drop_idx(self):
         if self.drop is None:
             return None
         elif isinstance(self.drop, str):
-            if self.drop == 'first':
+            if self.drop == "first":
                 return np.zeros(len(self.categories_), dtype=object)
-            elif self.drop == 'if_binary':
-                return np.array([0 if len(cats) == 2 else None
-                                for cats in self.categories_], dtype=object)
+            elif self.drop == "if_binary":
+                return np.array(
+                    [0 if len(cats) == 2 else None for cats in self.categories_],
+                    dtype=object,
+                )
             else:
                 msg = (
                     "Wrong input for parameter `drop`. Expected "
                     "'first', 'if_binary', None or array of objects, got {}"
-                    )
+                )
                 raise ValueError(msg.format(type(self.drop)))
 
         else:
@@ -366,16 +394,19 @@ class OneHotEncoder(_BaseEncoder):
                 msg = (
                     "Wrong input for parameter `drop`. Expected "
                     "'first', 'if_binary', None or array of objects, got {}"
-                    )
+                )
                 raise ValueError(msg.format(type(drop_array)))
             if droplen != len(self.categories_):
-                msg = ("`drop` should have length equal to the number "
-                       "of features ({}), got {}")
+                msg = (
+                    "`drop` should have length equal to the number "
+                    "of features ({}), got {}"
+                )
                 raise ValueError(msg.format(len(self.categories_), droplen))
             missing_drops = []
             drop_indices = []
-            for col_idx, (val, cat_list) in enumerate(zip(drop_array,
-                                                          self.categories_)):
+            for col_idx, (val, cat_list) in enumerate(
+                zip(drop_array, self.categories_)
+            ):
                 if not is_scalar_nan(val):
                     drop_idx = np.where(cat_list == val)[0]
                     if drop_idx.size:  # found drop idx
@@ -393,12 +424,18 @@ class OneHotEncoder(_BaseEncoder):
                     missing_drops.append((col_idx, val))
 
             if any(missing_drops):
-                msg = ("The following categories were supposed to be "
-                       "dropped, but were not found in the training "
-                       "data.\n{}".format(
-                           "\n".join(
-                                ["Category: {}, Feature: {}".format(c, v)
-                                    for c, v in missing_drops])))
+                msg = (
+                    "The following categories were supposed to be "
+                    "dropped, but were not found in the training "
+                    "data.\n{}".format(
+                        "\n".join(
+                            [
+                                "Category: {}, Feature: {}".format(c, v)
+                                for c, v in missing_drops
+                            ]
+                        )
+                    )
+                )
                 raise ValueError(msg)
             return np.array(drop_indices, dtype=object)
 
@@ -418,10 +455,10 @@ class OneHotEncoder(_BaseEncoder):
         Returns
         -------
         self
+            Fitted encoder.
         """
         self._validate_keywords()
-        self._fit(X, handle_unknown=self.handle_unknown,
-                  force_all_finite='allow-nan')
+        self._fit(X, handle_unknown=self.handle_unknown, force_all_finite="allow-nan")
         self.drop_idx_ = self._compute_drop_idx()
         return self
 
@@ -468,8 +505,13 @@ class OneHotEncoder(_BaseEncoder):
         """
         check_is_fitted(self)
         # validation of X happens in _check_X called by _transform
-        X_int, X_mask = self._transform(X, handle_unknown=self.handle_unknown,
-                                        force_all_finite='allow-nan')
+        warn_on_unknown = self.handle_unknown == "ignore" and self.drop is not None
+        X_int, X_mask = self._transform(
+            X,
+            handle_unknown=self.handle_unknown,
+            force_all_finite="allow-nan",
+            warn_on_unknown=warn_on_unknown,
+        )
 
         n_samples, n_features = X_int.shape
 
@@ -502,13 +544,15 @@ class OneHotEncoder(_BaseEncoder):
 
         indptr = np.empty(n_samples + 1, dtype=int)
         indptr[0] = 0
-        np.sum(X_mask, axis=1, out=indptr[1:])
+        np.sum(X_mask, axis=1, out=indptr[1:], dtype=indptr.dtype)
         np.cumsum(indptr[1:], out=indptr[1:])
         data = np.ones(indptr[-1])
 
-        out = sparse.csr_matrix((data, indices, indptr),
-                                shape=(n_samples, feature_indices[-1]),
-                                dtype=self.dtype)
+        out = sparse.csr_matrix(
+            (data, indices, indptr),
+            shape=(n_samples, feature_indices[-1]),
+            dtype=self.dtype,
+        )
         if not self.sparse:
             return out.toarray()
         else:
@@ -518,8 +562,10 @@ class OneHotEncoder(_BaseEncoder):
         """
         Convert the data back to the original representation.
 
-        In case unknown categories are encountered (all zeros in the
-        one-hot encoding), ``None`` is used to represent this category.
+        When unknown categories are encountered (all zeros in the
+        one-hot encoding), ``None`` is used to represent this category. If the
+        feature with the unknown category has a dropped category, the dropped
+        category will be its inverse.
 
         Parameters
         ----------
@@ -533,13 +579,12 @@ class OneHotEncoder(_BaseEncoder):
             Inverse transformed array.
         """
         check_is_fitted(self)
-        X = check_array(X, accept_sparse='csr')
+        X = check_array(X, accept_sparse="csr")
 
         n_samples, _ = X.shape
         n_features = len(self.categories_)
         if self.drop_idx_ is None:
-            n_transformed_features = sum(len(cats)
-                                         for cats in self.categories_)
+            n_transformed_features = sum(len(cats) for cats in self.categories_)
         else:
             n_transformed_features = sum(
                 len(cats) - 1 if to_drop is not None else len(cats)
@@ -547,8 +592,9 @@ class OneHotEncoder(_BaseEncoder):
             )
 
         # validate shape of passed X
-        msg = ("Shape of the passed X data is not correct. Expected {0} "
-               "columns, got {1}.")
+        msg = (
+            "Shape of the passed X data is not correct. Expected {0} columns, got {1}."
+        )
         if X.shape[1] != n_transformed_features:
             raise ValueError(msg.format(n_transformed_features, X.shape[1]))
 
@@ -573,15 +619,20 @@ class OneHotEncoder(_BaseEncoder):
                 X_tr[:, i] = self.categories_[i][self.drop_idx_[i]]
                 j += n_categories
                 continue
-            sub = X[:, j:j + n_categories]
+            sub = X[:, j : j + n_categories]
             # for sparse X argmax returns 2D matrix, ensure 1D array
             labels = np.asarray(sub.argmax(axis=1)).flatten()
             X_tr[:, i] = cats[labels]
-            if self.handle_unknown == 'ignore':
+            if self.handle_unknown == "ignore":
                 unknown = np.asarray(sub.sum(axis=1) == 0).flatten()
                 # ignored unknown categories: we have a row of all zero
                 if unknown.any():
-                    found_unknown[i] = unknown
+                    # if categories were dropped then unknown categories will
+                    # be mapped to the dropped category
+                    if self.drop_idx_ is None or self.drop_idx_[i] is None:
+                        found_unknown[i] = unknown
+                    else:
+                        X_tr[unknown, i] = self.categories_[i][self.drop_idx_[i]]
             else:
                 dropped = np.asarray(sub.sum(axis=1) == 0).flatten()
                 if dropped.any():
@@ -590,12 +641,11 @@ class OneHotEncoder(_BaseEncoder):
                         raise ValueError(
                             f"Samples {all_zero_samples} can not be inverted "
                             "when drop=None and handle_unknown='error' "
-                            "because they contain all zeros")
+                            "because they contain all zeros"
+                        )
                     # we can safely assume that all of the nulls in each column
                     # are the dropped value
-                    X_tr[dropped, i] = self.categories_[i][
-                        self.drop_idx_[i]
-                    ]
+                    X_tr[dropped, i] = self.categories_[i][self.drop_idx_[i]]
 
             j += n_categories
 
@@ -610,9 +660,12 @@ class OneHotEncoder(_BaseEncoder):
 
         return X_tr
 
+    @deprecated(
+        "get_feature_names is deprecated in 1.0 and will be removed "
+        "in 1.2. Please use get_feature_names_out instead."
+    )
     def get_feature_names(self, input_features=None):
-        """
-        Return feature names for output features.
+        """Return feature names for output features.
 
         Parameters
         ----------
@@ -628,23 +681,55 @@ class OneHotEncoder(_BaseEncoder):
         check_is_fitted(self)
         cats = self.categories_
         if input_features is None:
-            input_features = ['x%d' % i for i in range(len(cats))]
+            input_features = ["x%d" % i for i in range(len(cats))]
         elif len(input_features) != len(self.categories_):
             raise ValueError(
                 "input_features should have length equal to number of "
-                "features ({}), got {}".format(len(self.categories_),
-                                               len(input_features)))
+                "features ({}), got {}".format(
+                    len(self.categories_), len(input_features)
+                )
+            )
 
         feature_names = []
         for i in range(len(cats)):
-            names = [
-                input_features[i] + '_' + str(t) for t in cats[i]]
+            names = [input_features[i] + "_" + str(t) for t in cats[i]]
             if self.drop_idx_ is not None and self.drop_idx_[i] is not None:
                 names.pop(self.drop_idx_[i])
             feature_names.extend(names)
 
         return np.array(feature_names, dtype=object)
 
+    def get_feature_names_out(self, input_features=None):
+        """Get output feature names for transformation.
+
+        Parameters
+        ----------
+        input_features : array-like of str or None, default=None
+            Input features.
+
+            - If `input_features` is `None`, then `feature_names_in_` is
+              used as feature names in. If `feature_names_in_` is not defined,
+              then names are generated: `[x0, x1, ..., x(n_features_in_)]`.
+            - If `input_features` is an array-like, then `input_features` must
+              match `feature_names_in_` if `feature_names_in_` is defined.
+
+        Returns
+        -------
+        feature_names_out : ndarray of str objects
+            Transformed feature names.
+        """
+        check_is_fitted(self)
+        cats = self.categories_
+        input_features = _check_feature_names_in(self, input_features)
+
+        feature_names = []
+        for i in range(len(cats)):
+            names = [input_features[i] + "_" + str(t) for t in cats[i]]
+            if self.drop_idx_ is not None and self.drop_idx_[i] is not None:
+                names.pop(self.drop_idx_[i])
+            feature_names.extend(names)
+        return np.asarray(feature_names, dtype=object)
+
 
 class OrdinalEncoder(_BaseEncoder):
     """
@@ -699,6 +784,17 @@ class OrdinalEncoder(_BaseEncoder):
         the features in X and corresponding with the output of ``transform``).
         This does not include categories that weren't seen during ``fit``.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 1.0
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
     See Also
     --------
     OneHotEncoder : Performs a one-hot encoding of categorical features.
@@ -726,9 +822,14 @@ class OrdinalEncoder(_BaseEncoder):
            ['Female', 2]], dtype=object)
     """
 
-    @_deprecate_positional_args
-    def __init__(self, *, categories='auto', dtype=np.float64,
-                 handle_unknown='error', unknown_value=None):
+    def __init__(
+        self,
+        *,
+        categories="auto",
+        dtype=np.float64,
+        handle_unknown="error",
+        unknown_value=None,
+    ):
         self.categories = categories
         self.dtype = dtype
         self.handle_unknown = handle_unknown
@@ -749,42 +850,66 @@ class OrdinalEncoder(_BaseEncoder):
 
         Returns
         -------
-        self
+        self : object
+            Fitted encoder.
         """
         handle_unknown_strategies = ("error", "use_encoded_value")
         if self.handle_unknown not in handle_unknown_strategies:
             raise ValueError(
-                f"handle_unknown should be either 'error' or "
+                "handle_unknown should be either 'error' or "
                 f"'use_encoded_value', got {self.handle_unknown}."
             )
 
-        if self.handle_unknown == 'use_encoded_value':
+        if self.handle_unknown == "use_encoded_value":
             if is_scalar_nan(self.unknown_value):
-                if np.dtype(self.dtype).kind != 'f':
+                if np.dtype(self.dtype).kind != "f":
                     raise ValueError(
-                        f"When unknown_value is np.nan, the dtype "
+                        "When unknown_value is np.nan, the dtype "
                         "parameter should be "
                         f"a float dtype. Got {self.dtype}."
                     )
             elif not isinstance(self.unknown_value, numbers.Integral):
-                raise TypeError(f"unknown_value should be an integer or "
-                                f"np.nan when "
-                                f"handle_unknown is 'use_encoded_value', "
-                                f"got {self.unknown_value}.")
+                raise TypeError(
+                    "unknown_value should be an integer or "
+                    "np.nan when "
+                    "handle_unknown is 'use_encoded_value', "
+                    f"got {self.unknown_value}."
+                )
         elif self.unknown_value is not None:
-            raise TypeError(f"unknown_value should only be set when "
-                            f"handle_unknown is 'use_encoded_value', "
-                            f"got {self.unknown_value}.")
+            raise TypeError(
+                "unknown_value should only be set when "
+                "handle_unknown is 'use_encoded_value', "
+                f"got {self.unknown_value}."
+            )
 
-        self._fit(X)
+        # `_fit` will only raise an error when `self.handle_unknown="error"`
+        self._fit(X, handle_unknown=self.handle_unknown, force_all_finite="allow-nan")
 
-        if self.handle_unknown == 'use_encoded_value':
+        if self.handle_unknown == "use_encoded_value":
             for feature_cats in self.categories_:
                 if 0 <= self.unknown_value < len(feature_cats):
-                    raise ValueError(f"The used value for unknown_value "
-                                     f"{self.unknown_value} is one of the "
-                                     f"values already used for encoding the "
-                                     f"seen categories.")
+                    raise ValueError(
+                        "The used value for unknown_value "
+                        f"{self.unknown_value} is one of the "
+                        "values already used for encoding the "
+                        "seen categories."
+                    )
+
+        # stores the missing indices per category
+        self._missing_indices = {}
+        for cat_idx, categories_for_idx in enumerate(self.categories_):
+            for i, cat in enumerate(categories_for_idx):
+                if is_scalar_nan(cat):
+                    self._missing_indices[cat_idx] = i
+                    continue
+
+        if np.dtype(self.dtype).kind != "f" and self._missing_indices:
+            raise ValueError(
+                "There are missing values in features "
+                f"{list(self._missing_indices)}. For OrdinalEncoder to "
+                "passthrough missing values, the dtype parameter must be a "
+                "float"
+            )
 
         return self
 
@@ -802,11 +927,17 @@ class OrdinalEncoder(_BaseEncoder):
         X_out : ndarray of shape (n_samples, n_features)
             Transformed input.
         """
-        X_int, X_mask = self._transform(X, handle_unknown=self.handle_unknown)
+        X_int, X_mask = self._transform(
+            X, handle_unknown=self.handle_unknown, force_all_finite="allow-nan"
+        )
         X_trans = X_int.astype(self.dtype, copy=False)
 
+        for cat_idx, missing_idx in self._missing_indices.items():
+            X_missing_mask = X_int[:, cat_idx] == missing_idx
+            X_trans[X_missing_mask, cat_idx] = np.nan
+
         # create separate category for unknown values
-        if self.handle_unknown == 'use_encoded_value':
+        if self.handle_unknown == "use_encoded_value":
             X_trans[~X_mask] = self.unknown_value
         return X_trans
 
@@ -816,7 +947,7 @@ class OrdinalEncoder(_BaseEncoder):
 
         Parameters
         ----------
-        X : {array-like, sparse matrix} of shape (n_samples, n_features)
+        X : array-like of shape (n_samples, n_encoded_features)
             The transformed data.
 
         Returns
@@ -825,14 +956,15 @@ class OrdinalEncoder(_BaseEncoder):
             Inverse transformed array.
         """
         check_is_fitted(self)
-        X = check_array(X, accept_sparse='csr')
+        X = check_array(X, force_all_finite="allow-nan")
 
         n_samples, _ = X.shape
         n_features = len(self.categories_)
 
         # validate shape of passed X
-        msg = ("Shape of the passed X data is not correct. Expected {0} "
-               "columns, got {1}.")
+        msg = (
+            "Shape of the passed X data is not correct. Expected {0} columns, got {1}."
+        )
         if X.shape[1] != n_features:
             raise ValueError(msg.format(n_features, X.shape[1]))
 
@@ -843,11 +975,16 @@ class OrdinalEncoder(_BaseEncoder):
         found_unknown = {}
 
         for i in range(n_features):
-            labels = X[:, i].astype('int64', copy=False)
-            if self.handle_unknown == 'use_encoded_value':
+            labels = X[:, i].astype("int64", copy=False)
+
+            # replace values of X[:, i] that were nan with actual indices
+            if i in self._missing_indices:
+                X_i_mask = _get_mask(X[:, i], np.nan)
+                labels[X_i_mask] = self._missing_indices[i]
+
+            if self.handle_unknown == "use_encoded_value":
                 unknown_labels = labels == self.unknown_value
-                X_tr[:, i] = self.categories_[i][np.where(
-                    unknown_labels, 0, labels)]
+                X_tr[:, i] = self.categories_[i][np.where(unknown_labels, 0, labels)]
                 found_unknown[i] = unknown_labels
             else:
                 X_tr[:, i] = self.categories_[i][labels]
